/*
Software Serial Examples.
No change necessary.
 */
#include <SoftwareSerial.h>

SoftwareSerial esp8266(11, 12);

boolean buttonState = LOW;
boolean oldbuttonState = LOW;
byte ButtonCount = 0;

byte buffer[128];

byte MQTTCONNECT = 16;
byte MQTTPUBLISH = 48;

// {48, 40, 0, 35, 47, 'E', 'l', 'e', 'k', 't', 'o', 'r', 'M', 'y', 'J', 'o', 'u', 'r', 'n', 'e', 'y', 'I', 'o', 'T', 47, 'T', 'e', 's', 't', 'T', 'o', 'p', 'i', 'c', 47, 't', 'e', 's', 't', 'T', 'x', 't'};

byte payload[7] = {'b','u','t','t','o','n','0'};



// Send MQTT-connection-request to MQTT-Server via TCP
int MQTTClient_Connect(String clientid) 
{
    int result = 0;

    // fill the buffer with data to be sent for connection request
    int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

    // for MQTT version 3.1.1 you always have to sent the following bytes
    byte d[7] = { 0, 4, 77, 81, 84, 84, 4};

    // fill the buffer with these bytes
    for (int j = 0 ;j < 7; j++) 
    {
        buffer[BufferIndex++] = d[j];
    }

    // next byte position is filled with flags, we only use the clean session flag
    buffer[BufferIndex++] = 0x02;

    // next two bytes are keep alive time
    buffer[BufferIndex++] = 0;
    buffer[BufferIndex++] = 180;

    // then write the client-ID in the buffer, with length bytes at the beginning
    BufferIndex = MQTTClient_WriteStringInBuffer(clientid, buffer, BufferIndex);

    // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
    result = MQTTClient_Send(MQTTCONNECT, buffer, BufferIndex - 2); 
  
    return result;  
}



// Send MQTT-Publish-message to MQTT-Server via TCP
// the byte array payload contains the payload data, plength is the amount of payload bytes
int MQTTClient_Publish(String topic, byte payload[], int plength) 
{
    // fill the buffer with data to be sent for publish message
    int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

    // write the topic in the buffer, with topic length bytes at the beginning and offset = 2
    BufferIndex = MQTTClient_WriteStringInBuffer(topic, buffer, BufferIndex);

    // then copy the payload bytes in the buffer
    for (int i = 0; i  < plength; i++)
    {
        buffer[BufferIndex++] = payload[i];
    }

    // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
    return MQTTClient_Send(MQTTPUBLISH, buffer, BufferIndex - 2);
}




// send out a MQTT message from client to server
// send the buffer via TCP/IP, put message type and remaining length byte at the beginning
// buf contains the bytes to be sent out, with 2 bytes left free at the beginning
// RemainingLengthBytes is the number of bytes, without the 2 bytes at the beginning
int MQTTClient_Send(byte MessageType, byte buf[], int RemainingLengthByte)
{   
    buf[0] = MessageType;        
    buf[1] = RemainingLengthByte & 255;
      
    // send out all bytes via TCP/IP
    int result = TCPClient_Send(buf, RemainingLengthByte + 2);

    return result;
}





// this function writes the bytes (ASCII) of a string in the buffer buf, with a given offset, and returns a new offset/index back
int MQTTClient_WriteStringInBuffer(String MString, byte buf[], int offset) 
{
  
    int Slength = MString.length();
    
    // copy the bytes of this array in the buffer, with the given offset, and leave 2 bytes free at the beginning
    for (int j = 0; j < Slength; j++)
    {
        buf[offset + 2 + j] = MString[j];
    }

    // fill the lenght of the string in the first 2 bytes (string length is lower than 128 in that case)
    buf[offset] = 0;
    buf[offset + 1] = Slength;

    // return new offset/index after the string
    return offset + 2 + Slength;
}



// Closes TCP/IP-connection to TCP/IP-Server

int TCPClient_Close()
{
    int Result = 0;

    
    // send AT-command to ESP8266 to connect to TCP/IP-server
    esp8266.println("AT+CIPCLOSE");

    
    Result = 1;   
    

    return Result;
}




// Establish TCP/IP-connection to TCP/IP-Server

int TCPClient_Connect(String IPAddressString, int Port)
{
    int Result = 0;

    
    // send AT-command to ESP8266 to connect to TCP/IP-server
    esp8266.println("AT+CIPSTART=\"TCP\",\"" + IPAddressString + "\"," + String(Port) + "\r\n");

    delay(2000);

    Result = 1;   
    

    return Result;
}




int TCPClient_Send(byte data[], int len)
{
     String Send = "AT+CIPSEND=" + String(len) + "\r\n";
     
     esp8266.println(Send);
     
     delay(1000);
          
     esp8266.write(data,len);

     return 1;
}




String txtSSID = "YOUR SSID";
String txtPassword = "YOUR PASSWORD"; 

void setup()
{
  
  pinMode(9, OUTPUT);
  pinMode(8, INPUT_PULLUP);

  boolean buttonState = LOW;
  boolean oldbuttonState = LOW;
  
  // Open serial communications and wait for port to open:
  Serial.begin(19200);
  // set the data rate for the SoftwareSerial port
  esp8266.begin(19200);
  
  //send first AT Command to see if ESP8266-Module responds
  //esp8266.println("AT");

  ButtonCount = 0;

  esp8266.println("AT+RST");

  delay(2000);

  // send AT-command to ESP8266 to put ESP8266 in client mode
  esp8266.println("AT+CWMODE=1");

  delay(2000);
  
  // connect to WiFi network
  esp8266.println ("AT+CWJAP=\"" + txtSSID + "\",\"" + txtPassword + "\"");

  // LED on
  digitalWrite(9, HIGH);
}






void loop() // run over and over
{
  if (esp8266.available())
  {
    Serial.write(esp8266.read());
   
  }

    
  if (Serial.available())
  {
    esp8266.write(Serial.read());
    
  }

 
  buttonState = digitalRead(8);

  // button is pressed
  if (buttonState == LOW && oldbuttonState == HIGH)
  {
      payload[6] = 48 + ButtonCount;

      TCPClient_Close();
      delay(1000);

      // HiveMQ Testserver
      TCPClient_Connect("52.29.193.216", 1883);

      MQTTClient_Connect("elektorIoT");
      delay(1000);
      
      MQTTClient_Publish("/ElektorMyJourneyIoT/TestTopic/test", payload, 7);

      ButtonCount++;
      if (ButtonCount > 9) ButtonCount = 0;
  }

  
  oldbuttonState = buttonState;
  
  
}


